using Autofac.Extensions.DependencyInjection;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using Serilog;
using System;
using System.IO;

namespace QueryPlatform.Api
{
    public class Program
    {
        static Program()
        {
            string env = Environment.GetEnvironmentVariable("ASPNETCORE_ENVIRONMENT");
            BaseDirectory = AppDomain.CurrentDomain.BaseDirectory;
            Configuration = new ConfigurationBuilder()
                .SetBasePath(BaseDirectory)
                .AddJsonFile($"appsettings.json", optional: false, reloadOnChange: true)
                .AddJsonFile($"appsettings.{env}.json", optional: true, reloadOnChange: true)
                .AddJsonFile($"secret.json", optional: true, reloadOnChange: true)
                .AddXmlFile($"overview.xml", true, true)
                .AddEnvironmentVariables()
                .Build();
        }

        public static IConfiguration Configuration { get; }
        public static string BaseDirectory { get; }

        public static void Main(string[] args)
        {
            Log.Logger = CreateSerilogLogger();

            Log.Information("Starting web host");
            CreateHostBuilder(args).Build().Run();
        }

        public static IHostBuilder CreateHostBuilder(string[] args)
        {
            var urls = Configuration.GetValue("ListenUrls", "http://*:56900");

            return Host.CreateDefaultBuilder(args)
                .UseServiceProviderFactory(new AutofacServiceProviderFactory())
                .ConfigureWebHostDefaults(webBuilder =>
                {
                    webBuilder
                        .UseUrls(urls.Split(','))
                        .UseStartup<Startup>();
                })
                .ConfigureAppConfiguration((context, config) =>
                {
                    config.AddConfiguration(Configuration);
                })
                .UseContentRoot(BaseDirectory)
                .UseSerilog();
        }

        private static Serilog.ILogger CreateSerilogLogger()
        {
            var basePath = Path.Combine(BaseDirectory, "logs");
            return new LoggerConfiguration()
                .ReadFrom.Configuration(Configuration)
                //.MinimumLevel.Verbose()
                .Enrich.FromLogContext()
                .WriteTo.Console()
                .WriteTo.File($"{basePath}\\.log", rollingInterval: RollingInterval.Day, retainedFileCountLimit: null)
                .CreateLogger();
        }
    }
}
